@@ -211,9 +211,19 @@ public final class BasicVector_UnsignedByte8 extends AbstractVector
   @Override
   public void fill(LispObject obj)
   {
-    byte b = coerceLispObjectToJavaByte(obj);
+    if (!(obj instanceof Fixnum)) {
+      type_error(obj, Symbol.FIXNUM);
+      // Not reached.
+      return;
+    }
+    int n = ((Fixnum) obj).value;
+    if (n < 0 || n > 255) {
+      type_error(obj, UNSIGNED_BYTE_8);
+      // Not reached.
+      return;
+    }
     for (int i = capacity; i-- > 0;)
-      elements[i] = b;
+      elements[i] = (byte) n;
   }
 
   @Override
