@@ -147,9 +147,12 @@
         ((xp::xp-structure-p stream)
          (let ((s (sys::%write-to-string object)))
            (xp::write-string++ s stream 0 (length s))))
+	((functionp object)
+	  (print-object object stream))
         (t
          (%output-object object stream))))
 
+
 ;;;; circularity detection stuff
 
 ;;; When *PRINT-CIRCLE* is T, this gets bound to a hash table that
